import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.NoSuchFileException;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.Stack;

import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.TextInputDialog;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Text;
import javafx.stage.Stage;

class Node implements Comparable<Node>
{
    double val;
    String key;
    Node parent;
    Node left;
    Node right;

    public Node(double val2, String key)
    {
        this.val = val2;
        this.key = key;
        this.left = null;
        this.right = null;
        this.parent = null;
    }

    public Node(double d)
    {
        this.val = d;
        this.key = null;
        this.left = null;
        this.right = null;
        this.parent = null;
    }

	public int compareTo(Node n) {
		if(this.val>n.val) return 1;
		else if (this.val<n.val) return -1;
		else return 0;
	}

    public boolean isLeaf() {
    	return left==null && right==null;
    }
}

class Data implements Comparable<Data>
{
    double val;
    String key;

	public Data(String key, double d) {
        this.val = d;
        this.key = key;
	}

	public int compareTo(Data n) {
		if(this.val>n.val) return 1;
		else if (this.val<n.val) return -1;
		else return 0;
	}

    public static Node readFile(String path) {
        List<String> lines = null;
        try {
            lines = Files.readAllLines(Paths.get(path));
        } catch (IOException e) {
        	if(e instanceof NoSuchFileException) {
        		Alert alert = new Alert(AlertType.ERROR);
        		alert.setTitle("Erreur");
        		alert.setHeaderText("Chemin invalide");
        		alert.setContentText("Le fichier <"+path+"> n'existe pas !");

        		alert.showAndWait();

        		System.exit(1);
        	}

        	else e.printStackTrace();
        }

        return newTree(lines);
    }

    public static Node testTree() {
    	return newTree(Arrays.asList("Nom,Valeur","A (valeur:1),1","B (valeur:2),2","C (valeur:3),3","D (valeur:4),4","E (valeur:5),5"));
    }

    public static Node newTree(List<String> lines) {
    	List<Data> l = new ArrayList<Data>();
        for (int i = 1; i < lines.size(); i++) {
        	String[] split = lines.get(i).split(",");
        	if (split.length!=2 || !isNumber(split[1])) {
        		Alert alert = new Alert(AlertType.ERROR);
        		alert.setTitle("Erreur");
        		alert.setHeaderText("Format invalide");
        		alert.setContentText("La ligne "+i+", <"+lines.get(i)+"> est de format incorrect \n(Rappel: <Nom,Valeur(nombre au format int ou double)>)");

        		alert.showAndWait();

        		System.exit(1);
        	}
        	Data d = new Data(split[0],Double.parseDouble(split[1]));
        	l.add(d);
        }

    	return TreeMap.buildTree(l);
    }

	private static boolean isNumber(String string) {
	    if (string == null) return false;
	    try { Double.parseDouble(string); }
	    catch (NumberFormatException nfe) { return false; }
	    return true;
	}
}

class TreeMap
{

	public static Node buildTree(List<Data> l) {

		Collections.sort(l, Collections.reverseOrder());
    	Stack<Node> nodes = new Stack<Node>();

    	for(Data d:l) {
    		Node t = new Node(d.val,d.key);
    		nodes.add(t);
    	}

		while(nodes.size()>1) {
    		Node t1 = nodes.pop();
    		Node t2 = nodes.pop();
    		Node t3 = new Node(t1.val+t2.val);
    		t3.left = t1;
    		t3.right = t2;
    		nodes.add(t3);
    		Collections.sort(nodes, Collections.reverseOrder());
    	}

		return nodes.firstElement();
	}

    public static Pane listRectangles(Rectangle r, boolean o, Node t) {
    	//o = orientation, true = hor, false = ver
    	if(t.isLeaf()) return new Pane(r,new Text(r.getWidth()/2-t.key.length()*2,r.getHeight()/2,t.key));
    	else {
    		double sum = t.left.val+t.right.val;
    		double s1 = (double)t.left.val/(double)sum;
    		double s2 = (double)t.right.val/(double)sum;
    		Rectangle r1 = new Rectangle(
    						o?r.getWidth()*s1:r.getWidth(),
    						!o?r.getHeight()*s1:r.getHeight(),
    						Color.color(Math.random(), Math.random(), Math.random())
    					);
    		Rectangle r2 = new Rectangle(
    						o?r.getWidth()*s2:r.getWidth(),
    						!o?r.getHeight()*s2:r.getHeight(),
    						Color.color(Math.random(), Math.random(), Math.random())
    					);
    		r1.setStroke(Color.BLACK);r2.setStroke(Color.BLACK);

    		Pane l1 = listRectangles(r1,!o,t.left);
    		Pane l2 = listRectangles(r2,!o,t.right);

    		return o?new HBox(l1,l2):new VBox(l1,l2);
    	}
    }

    public static void drawMap(Node root) {
    	Pane p = listRectangles(new Rectangle(TreeMapViewer.TAILLE,TreeMapViewer.TAILLE),false,root);
    	TreeMapViewer.appRoot.getChildren().add(p);
    	TreeMapViewer.appRoot.setStyle("-fx-background-color: #000000");
    }

}

public class TreeMapViewer extends Application{

	public static double TAILLE = 800;
	public static Pane appRoot = new Pane();
    public static Stage stage;

	@Override
	public void start(Stage primaryStage) throws Exception {
		primaryStage = initStage(primaryStage);
		primaryStage.setResizable(false);

		TreeMap.drawMap(initTree());

    	primaryStage.show();
	}

    public Stage initStage(Stage stage) {
        Scene scene = new Scene(appRoot,TAILLE,TAILLE);

        stage.setTitle("TreeMap Viewer");
        stage.setScene(scene);
        stage.setWidth(TAILLE);
        stage.setHeight(TAILLE);

        stage.setResizable(false);

        return stage;
    }

    public static Node initTree() {
    	TextInputDialog dialog = new TextInputDialog();
    	dialog.setTitle("TreeMap Viewer");
    	dialog.setHeaderText("Voulez-vous charger un fichier pour créer une TreeMap ?");
    	dialog.setContentText("Rentrer le chemin du fichier ou laisser vide pour l'exemple par défaut:");

    	Optional<String> result = dialog.showAndWait();
    	if (result.isPresent()){
    		return result.get().length()>0?Data.readFile(result.get()):Data.testTree();
    	}
    	else System.exit(0);
    	return null;
    }

}
